@@ -27,7 +27,7 @@ import mx4j.MX4JSystemKeys;
 import mx4j.util.Utils;
 
 /**
- * @version $Revision: 1.24 $
+ * @version $Revision: 1.25 $
  */
 public class ObjectName implements QueryExp, Serializable
 {
@@ -603,6 +603,16 @@ public class ObjectName implements QueryExp, Serializable
 
       if (value.trim().startsWith("\""))
       {
+         // strip leading and trailing spaces
+         value = value.trim();
+
+         // check value has quotes at start and end
+         if (value.length() < 2) return false;
+         if (value.charAt(value.length()-1) != '"') return false;
+
+         // check final quote is not escaped
+         if (countBackslashesBackwards(value, value.length()-1) % 2 == 1) return false;
+
          // Unquote the value
          value = value.substring(1, value.length() - 1);
 
