@@ -46,7 +46,7 @@ import mx4j.log.Log;
 import mx4j.log.Logger;
 
 /**
- * @version $Revision: 1.26 $
+ * @version $Revision: 1.27 $
  */
 public class MLet extends URLClassLoader implements MLetMBean, MBeanRegistration, Externalizable
 {
@@ -104,7 +104,7 @@ public class MLet extends URLClassLoader implements MLetMBean, MBeanRegistration
    public ObjectName preRegister(MBeanServer server, ObjectName name) throws Exception
    {
       this.server = server;
-      objectName = name == null ? new ObjectName(this.server.getDefaultDomain(), "service", "MLet") : name;
+      objectName = name == null ? new ObjectName(this.server.getDefaultDomain(), "type", "MLet") : name;
       Logger logger = getLogger();
       if (logger.isEnabledFor(Logger.TRACE)) logger.trace("MLet service " + objectName + " preRegistered successfully");
       return objectName;
