@@ -11,7 +11,7 @@ package javax.management.modelmbean;
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.ObjectInputStream;
-import java.util.ArrayList;
+import java.io.ObjectOutputStream;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
@@ -29,13 +29,13 @@ import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
 
 /**
- * @version $Revision: 1.28 $
+ * @version $Revision: 1.30 $
  */
 public class DescriptorSupport implements Descriptor
 {
    private static final long serialVersionUID = -6292969195866300415L;
 
-   private HashMap descriptor = new HashMap(20);
+   private HashMap descriptor;
    private transient HashMap fields = new HashMap(20);
 
    public DescriptorSupport()
@@ -48,7 +48,6 @@ public class DescriptorSupport implements Descriptor
       {
          throw new RuntimeOperationsException(new IllegalArgumentException("Number of Fields cannot be <= 0"));
       }
-      descriptor = new HashMap(initNumFields);
       fields = new HashMap(initNumFields);
    }
 
@@ -81,10 +80,9 @@ public class DescriptorSupport implements Descriptor
          for (int i = 0; i < pairs.length; ++i)
          {
             String pair = pairs[i];
-            if (pair == null)
-            {
-               throw new RuntimeOperationsException(new IllegalArgumentException("Illegal pair: " + pair));
-            }
+            // null or empty strings are to be ignored
+            if (pair == null || pair.length() == 0) continue;
+
             int equal = pair.indexOf('=');
             if (equal < 1)
             {
@@ -128,62 +126,79 @@ public class DescriptorSupport implements Descriptor
    public void setField(String name, Object value) throws RuntimeOperationsException
    {
       checkField(name, value);
-      descriptor.put(name, value);
-      fields.put(name.toLowerCase(), new ValueHolder(name, value));
+
+      // update field but keep the original name if an entry already exists
+      String lcase = name.toLowerCase();
+      ValueHolder holder = (ValueHolder) fields.get(lcase);
+      ValueHolder newHolder = new ValueHolder(holder == null ? name : holder.fieldName, value);
+      fields.put(lcase, newHolder);
    }
 
    public void removeField(String name)
    {
       if (name != null)
       {
-         ValueHolder holder = (ValueHolder)fields.remove(name.toLowerCase());
-         if (holder != null) descriptor.remove(holder.fieldName);
+         fields.remove(name.toLowerCase());
       }
    }
 
    public String[] getFieldNames()
    {
-      // Preserve the case of field names
-      String[] names = (String[])descriptor.keySet().toArray(new String[0]);
+      // Preserve the case of field names so use the ones from the values
+      String[] names = new String[fields.size()];
+      int x = 0;
+      for (Iterator i = fields.values().iterator(); i.hasNext();)
+      {
+         ValueHolder holder = (ValueHolder) i.next();
+         names[x++] = holder.fieldName;
+      }
       return names;
    }
 
    public Object[] getFieldValues(String[] names)
    {
+      // quick check for empty descriptor (which overrides all)
+      if (fields.isEmpty()) return new Object[0];
+
       if (names == null)
       {
          // All values must be returned
-         Object[] values = descriptor.values().toArray(new Object[0]);
-         return values;
+         Object[] list = new Object[fields.size()];
+         int x = 0;
+         for (Iterator i = fields.values().iterator(); i.hasNext();)
+         {
+            ValueHolder holder = (ValueHolder) i.next();
+            list[x++] = holder.fieldValue;
+         }
+         return list;
       }
 
-      ArrayList list = new ArrayList();
+      Object[] list = new Object[names.length];
       for (int i = 0; i < names.length; ++i)
       {
          try
          {
-            Object value = getFieldValue(names[i]);
-            list.add(value);
+            list[i] = getFieldValue(names[i]);
          }
          catch (RuntimeOperationsException x)
          {
-            list.add(null);
+            list[i] = null;
          }
       }
-      Object[] values = list.toArray(new Object[list.size()]);
-      return values;
+      return list;
    }
 
    public String[] getFields()
    {
-      ArrayList list = new ArrayList();
+      String[] values = new String[fields.size()];
       StringBuffer buffer = new StringBuffer();
       // Preserve the case of field names
-      for (Iterator i = descriptor.entrySet().iterator(); i.hasNext();)
+      int x = 0;
+      for (Iterator i = fields.values().iterator(); i.hasNext();)
       {
-         Map.Entry entry = (Map.Entry)i.next();
-         String key = (String)entry.getKey();
-         Object value = entry.getValue();
+         ValueHolder holder = (ValueHolder) i.next();
+         String key = holder.fieldName;
+         Object value = holder.fieldValue;
          buffer.setLength(0);
          buffer.append(key);
          buffer.append("=");
@@ -200,10 +215,9 @@ public class DescriptorSupport implements Descriptor
                buffer.append(")");
             }
          }
-         list.add(buffer.toString());
+         values[x++] = buffer.toString();
       }
-      String[] fields = (String[])list.toArray(new String[list.size()]);
-      return fields;
+      return values;
    }
 
    public void setFields(String[] names, Object[] values) throws RuntimeOperationsException
@@ -225,12 +239,10 @@ public class DescriptorSupport implements Descriptor
 
       try
       {
-         for (Iterator i = descriptor.entrySet().iterator(); i.hasNext();)
+         for (Iterator i = fields.values().iterator(); i.hasNext();)
          {
-            Map.Entry entry = (Map.Entry)i.next();
-            String name = (String)entry.getKey();
-            Object value = entry.getValue();
-            checkField(name, value);
+            ValueHolder holder = (ValueHolder) i.next();
+            checkField(holder.fieldName, holder.fieldValue);
          }
          return true;
       }
@@ -247,17 +259,16 @@ public class DescriptorSupport implements Descriptor
 
       try
       {
-         if (descriptor.size() != 0)
+         if (fields.size() != 0)
          {
-            for (Iterator i = descriptor.entrySet().iterator();
-                 i.hasNext();
+            for (Iterator i = fields.values().iterator(); i.hasNext();
                     )
             {
-               Map.Entry entry = (Map.Entry)i.next();
-               Object value = entry.getValue();
+               ValueHolder holder = (ValueHolder) i.next();
+               Object value = holder.fieldValue;
                String valstr = toXMLValueString(value);
                buf.append("<field name=\"");
-               buf.append(entry.getKey());
+               buf.append(holder.fieldName);
                buf.append("\" value=\"");
                buf.append(valstr);
                buf.append("\"></field>");
@@ -277,12 +288,12 @@ public class DescriptorSupport implements Descriptor
       StringBuffer buf = new StringBuffer();
       try
       {
-         if (descriptor.size() != 0)
+         if (fields.size() != 0)
          {
-            for (Iterator i = descriptor.entrySet().iterator(); i.hasNext();)
+            for (Iterator i = fields.values().iterator(); i.hasNext();)
             {
-               Map.Entry entry = (Map.Entry)i.next();
-               buf.append(entry.getKey()).append(" ").append(entry.getValue());
+               ValueHolder holder = (ValueHolder) i.next();
+               buf.append(holder.fieldName).append(" ").append(holder.fieldValue);
                if (i.hasNext())
                {
                   buf.append(",");
@@ -333,77 +344,117 @@ public class DescriptorSupport implements Descriptor
 
       boolean isValid = true;
 
-      if (name.equalsIgnoreCase("name"))
+      // checks relaxed to match JavaDoc
+      if (name.equalsIgnoreCase("name") || name.equalsIgnoreCase("descriptorType"))
       {
-         isValid = value != null && value instanceof String && ((String)value).length() != 0;
+         isValid = value instanceof String && ((String)value).length() != 0;
       }
-      else if (name.equalsIgnoreCase("descriptorType"))
+      else if (name.equalsIgnoreCase("class") ||
+              name.equalsIgnoreCase("role") ||
+              name.equalsIgnoreCase("getMethod") ||
+              name.equalsIgnoreCase("setMethod"))
       {
-         isValid = value != null && (value.toString().equalsIgnoreCase("MBean") ||
-                                     value.toString().equalsIgnoreCase("attribute") ||
-                                     value.toString().equalsIgnoreCase("operation") ||
-                                     value.toString().equalsIgnoreCase("notification"));
+         isValid = value instanceof String;
       }
-      else if (name.equalsIgnoreCase("role"))
+      else if (name.equalsIgnoreCase("persistPeriod") ||
+            name.equalsIgnoreCase("currencyTimeLimit") ||
+            name.equalsIgnoreCase("lastUpdatedTimeStamp") ||
+            name.equalsIgnoreCase("lastReturnedTimeStamp"))
       {
-         isValid = value != null && (value.equals("constructor") ||
-                                     value.equals("operation") ||
-                                     value.equals("getter") ||
-                                     value.equals("setter"));
-      }
-      else if (name.equalsIgnoreCase("persistPolicy"))
-      {
-         isValid = value != null && (value.toString().equalsIgnoreCase("Never") ||
-                                     value.toString().equalsIgnoreCase("OnTimer") ||
-                                     value.toString().equalsIgnoreCase("OnUpdate") ||
-                                     value.toString().equalsIgnoreCase("NoMoreOftenThan") ||
-                                     value.toString().equalsIgnoreCase("Always"));
-      }
-      else if (name.equalsIgnoreCase("persistPeriod"))
-      {
-         int v = objectToInt(value);
-         isValid = v >= -1;
-      }
-      else if (name.equalsIgnoreCase("currencyTimeLimit"))
-      {
-         int v = objectToInt(value);
-         isValid = v >= -1;
-      }
-      else if (name.equalsIgnoreCase("visibility"))
-      {
-         int v = objectToInt(value);
-         isValid = v >= 1 && v <= 4;
-      }
-      else if (name.equalsIgnoreCase("getMethod") || name.equalsIgnoreCase("setMethod"))
-      {
-         isValid = value != null && value.toString().trim().length() > 0;
-      }
-      else if (name.equalsIgnoreCase("protocolMap"))
-      {
-         isValid = value instanceof Descriptor;
-      }
-      else if (name.equalsIgnoreCase("lastUpdatedTimeStamp"))
-      {
-         long v = objectToLong(value);
-         isValid = v > 0;
-      }
-      else if (name.equalsIgnoreCase("severity"))
-      {
-         int v = objectToInt(value);
-         isValid = v >= 0 && v <= 6;
-      }
-      else if (name.equalsIgnoreCase("messageId"))
-      {
-         isValid = value != null;
+         if (value instanceof Number)
+         {
+            isValid = ((Number)value).longValue() >= -1;
+         }
+         else if (value instanceof String)
+         {
+            try
+            {
+               isValid = Long.parseLong((String)value) >= -1;
+            } catch (NumberFormatException e)
+            {
+               isValid = false;
+            }
+         }
+         else
+         {
+            isValid = false;
+         }
       }
       else if (name.equalsIgnoreCase("log"))
       {
-         isValid = value != null &&
-                   (value instanceof Boolean ||
-                    (value instanceof String && (value.toString().equalsIgnoreCase("true") ||
-                                                 value.toString().equalsIgnoreCase("false") ||
-                                                 value.toString().equalsIgnoreCase("t") ||
-                                                 value.toString().equalsIgnoreCase("f"))));
+         if (value instanceof String)
+         {
+            String s = (String) value;
+            isValid = "t".equalsIgnoreCase(s) || "true".equalsIgnoreCase(s) ||
+                  "f".equalsIgnoreCase(s) || "false".equalsIgnoreCase(s);
+         }
+         else
+         {
+            isValid = value instanceof Boolean;
+         }
+      }
+      else if (name.equalsIgnoreCase("visibility"))
+      {
+         if (value instanceof Number)
+         {
+            long l = ((Number)value).longValue();
+            isValid = l >= 1 && l <= 4;
+         }
+         else if (value instanceof String)
+         {
+            try
+            {
+               long l = Long.parseLong((String)value);
+               isValid = l >= 1 && l <= 4;
+            } catch (NumberFormatException e)
+            {
+               isValid = false;
+            }
+         }
+         else
+         {
+            isValid = false;
+         }
+      }
+      else if (name.equalsIgnoreCase("severity"))
+      {
+         if (value instanceof Number)
+         {
+            long l = ((Number)value).longValue();
+            isValid = l >= 0 && l <= 6;
+         }
+         else if (value instanceof String)
+         {
+            try
+            {
+               long l = Long.parseLong((String)value);
+               isValid = l >= 0 && l <= 6;
+            } catch (NumberFormatException e)
+            {
+               isValid = false;
+            }
+         }
+         else
+         {
+            isValid = false;
+         }
+      }
+      else if (name.equalsIgnoreCase("persistPolicy"))
+      {
+         if (value instanceof String)
+         {
+            String s = (String) value;
+            isValid =
+                  "OnUpdate".equalsIgnoreCase(s) ||
+                  "OnTimer".equalsIgnoreCase(s) ||
+                  "NoMoreOftenThan".equalsIgnoreCase(s) ||
+                  "Always".equalsIgnoreCase(s) ||
+                  "Never".equalsIgnoreCase(s);
+         }
+         else
+         {
+            isValid = false;
+         }
       }
 
       if (!isValid)
@@ -465,38 +516,6 @@ public class DescriptorSupport implements Descriptor
       return result.toString();
    }
 
-   private int objectToInt(Object obj) throws RuntimeOperationsException
-   {
-      try
-      {
-         return Integer.parseInt(obj.toString());
-      }
-      catch (NumberFormatException x)
-      {
-         throw new RuntimeOperationsException(new IllegalArgumentException("Illegal value '" + obj + "' for numeric field"));
-      }
-      catch (NullPointerException x)
-      {
-         throw new RuntimeOperationsException(new IllegalArgumentException("Illegal value '" + obj + "' for numeric field"));
-      }
-   }
-
-   private long objectToLong(Object obj) throws RuntimeOperationsException
-   {
-      try
-      {
-         return Long.parseLong(obj.toString());
-      }
-      catch (NumberFormatException x)
-      {
-         throw new RuntimeOperationsException(new IllegalArgumentException("Illegal value '" + obj + "' for numeric field"));
-      }
-      catch (NullPointerException x)
-      {
-         throw new RuntimeOperationsException(new IllegalArgumentException("Illegal value '" + obj + "' for numeric field"));
-      }
-   }
-
    private Object parseValueString(String value) throws XMLParseException
    {
       Object result = null;
@@ -578,9 +597,9 @@ public class DescriptorSupport implements Descriptor
 
    private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException
    {
-      stream.defaultReadObject();
-      this.fields = new HashMap(descriptor.size());
-      for (Iterator i = descriptor.entrySet().iterator(); i.hasNext();)
+      HashMap desc = (HashMap) stream.readFields().get("descriptor", null);
+      fields = new HashMap(desc.size());
+      for (Iterator i = desc.entrySet().iterator(); i.hasNext();)
       {
          Map.Entry entry = (Map.Entry)i.next();
          String name = (String)entry.getKey();
@@ -588,6 +607,18 @@ public class DescriptorSupport implements Descriptor
       }
    }
 
+   private void writeObject(ObjectOutputStream stream) throws IOException {
+      HashMap desc = new HashMap(fields.size());
+      for (Iterator i = fields.values().iterator(); i.hasNext();)
+      {
+         ValueHolder holder = (ValueHolder) i.next();
+         desc.put(holder.fieldName, holder.fieldValue);
+      }
+      ObjectOutputStream.PutField fields = stream.putFields();
+      fields.put("descriptor", desc);
+      stream.writeFields();
+   }
+
    private static class ValueHolder
    {
       private final String fieldName;
