@@ -23,12 +23,17 @@ import javax.management.MBeanOperationInfo;
 import javax.management.RuntimeOperationsException;
 
 /**
- * @version $Revision: 1.17 $
+ * @version $Revision: 1.18 $
  */
 // Change not needed, workaround to a TCK bug only to achieve TCK compliance
 // public class ModelMBeanInfoSupport extends MBeanInfo implements ModelMBeanInfo
 public class ModelMBeanInfoSupport extends MBeanInfo implements ModelMBeanInfo, Serializable
 {
+   private static final MBeanAttributeInfo[] EMPTY_ATTRS = {};
+   private static final MBeanConstructorInfo[] EMPTY_CTRS = {};
+   private static final MBeanNotificationInfo[] EMPTY_NOTIFICATIONS = {};
+   private static final MBeanOperationInfo[] EMPTY_OPS = {};
+
    private static final long serialVersionUID = -1935722590756516193L;
 
    private Descriptor modelMBeanDescriptor;
@@ -48,10 +53,10 @@ public class ModelMBeanInfoSupport extends MBeanInfo implements ModelMBeanInfo,
    public ModelMBeanInfoSupport(String className, String description, ModelMBeanAttributeInfo[] attributes, ModelMBeanConstructorInfo[] constructors, ModelMBeanOperationInfo[] operations, ModelMBeanNotificationInfo[] notifications, Descriptor mbeanDescriptor)
    {
       super(className, description, attributes, constructors, operations, notifications);
-      modelMBeanAttributes = attributes;
-      modelMBeanConstructors = constructors;
-      modelMBeanNotifications = notifications;
-      modelMBeanOperations = operations;
+      modelMBeanAttributes = attributes != null ? attributes : EMPTY_ATTRS;
+      modelMBeanConstructors = constructors != null ? constructors : EMPTY_CTRS;
+      modelMBeanNotifications = notifications != null ? notifications : EMPTY_NOTIFICATIONS;
+      modelMBeanOperations = operations != null ? operations : EMPTY_OPS;
       checkAndSetDescriptor(mbeanDescriptor);
    }
 
