@@ -26,7 +26,6 @@ import javax.management.MBeanException;
 import javax.management.MBeanInfo;
 import javax.management.MBeanNotificationInfo;
 import javax.management.MBeanRegistration;
-import javax.management.MBeanRegistrationException;
 import javax.management.MBeanServer;
 import javax.management.MalformedObjectNameException;
 import javax.management.Notification;
@@ -52,11 +51,32 @@ import mx4j.persist.PersisterMBean;
 import mx4j.util.Utils;
 
 /**
- * @version $Revision: 1.19 $
+ * @version $Revision: 1.22 $
  */
 public class RequiredModelMBean implements ModelMBean, MBeanRegistration, NotificationEmitter
 {
    private static final String OBJECT_RESOURCE_TYPE = "ObjectReference";
+   private static final ModelMBeanInfoSupport DEFAULT_INFO;
+   private static final ModelMBeanNotificationInfo GENERIC_INFO;
+   private static final ModelMBeanNotificationInfo ATTR_CHANGE_INFO;
+   static
+   {
+      String[] fields = new String[]{
+         "name=GENERIC",
+         "descriptorType=notification",
+         "log=T",
+         "severity=6",
+         "displayName=jmx.modelmbean.generic"};
+      GENERIC_INFO = new ModelMBeanNotificationInfo(new String[]{"jmx.modelmbean.generic"}, "GENERIC", "", new DescriptorSupport(fields));
+      fields = new String[]{
+         "name=ATTRIBUTE_CHANGE",
+         "descriptorType=notification",
+         "log=T",
+         "severity=6",
+         "displayName=jmx.attribute.change"};
+      ATTR_CHANGE_INFO = new ModelMBeanNotificationInfo(new String[]{"jmx.attribute.change"}, "ATTRIBUTE_CHANGE", "", new DescriptorSupport(fields));
+      DEFAULT_INFO = new ModelMBeanInfoSupport(RequiredModelMBean.class.getName(), null, null, null, null, null);
+   }
 
    private static final int ALWAYS_STALE = 1;
    private static final int NEVER_STALE = 2;
@@ -70,22 +90,12 @@ public class RequiredModelMBean implements ModelMBean, MBeanRegistration, Notifi
 
    private MBeanServer m_mbeanServer;
    private Object m_managedResource;
-   private boolean m_canBeRegistered;
    private ModelMBeanInfo m_modelMBeanInfo;
-   private NotificationBroadcasterSupport m_attributeChangeBroadcaster = new NotificationBroadcasterSupport();
    private NotificationBroadcasterSupport m_generalBroadcaster = new NotificationBroadcasterSupport();
 
    public RequiredModelMBean() throws MBeanException, RuntimeOperationsException
    {
-      try
-      {
-         load();
-      }
-      catch (Exception x)
-      {
-         Logger logger = getLogger();
-         logger.warn("Cannot restore previously saved status", x);
-      }
+      m_modelMBeanInfo = DEFAULT_INFO;
    }
 
    public RequiredModelMBean(ModelMBeanInfo info) throws MBeanException, RuntimeOperationsException
@@ -103,15 +113,9 @@ public class RequiredModelMBean implements ModelMBean, MBeanRegistration, Notifi
 
    public ObjectName preRegister(MBeanServer server, ObjectName name) throws Exception
    {
-      if (m_canBeRegistered)
-      {
-         m_mbeanServer = server;
-         return name;
-      }
-      else
-      {
-         throw new MBeanRegistrationException(new IllegalStateException("ModelMBean cannot be registered until setModelMBeanInfo has been called"));
-      }
+      if (name == null) throw new IllegalArgumentException("ObjectName is null");
+      m_mbeanServer = server;
+      return name;
    }
 
    public void postRegister(Boolean registrationDone)
@@ -136,15 +140,14 @@ public class RequiredModelMBean implements ModelMBean, MBeanRegistration, Notifi
 
    public void setModelMBeanInfo(ModelMBeanInfo modelMBeanInfo) throws MBeanException, RuntimeOperationsException
    {
+      if (m_mbeanServer != null) throw new RuntimeOperationsException(new IllegalStateException("setModelMBeanInfo cannot be called when ModelMBean is registered"));
       if (modelMBeanInfo == null) throw new RuntimeOperationsException(new IllegalArgumentException("ModelMBeanInfo cannot be null"));
       if (!isModelMBeanInfoValid(modelMBeanInfo)) throw new RuntimeOperationsException(new IllegalArgumentException("ModelMBeanInfo is invalid"));
 
-      m_modelMBeanInfo = (ModelMBeanInfo)modelMBeanInfo.clone();
+      m_modelMBeanInfo = new ModelMBeanInfoSupport(modelMBeanInfo);
 
       Logger logger = getLogger();
       if (logger.isEnabledFor(Logger.DEBUG)) logger.debug("ModelMBeanInfo successfully set to: " + m_modelMBeanInfo);
-      // Only now the MBean can be registered in the MBeanServer
-      m_canBeRegistered = true;
    }
 
    private boolean isModelMBeanInfoValid(ModelMBeanInfo info)
@@ -156,7 +159,6 @@ public class RequiredModelMBean implements ModelMBean, MBeanRegistration, Notifi
 
    public void setManagedResource(Object resource, String resourceType) throws MBeanException, RuntimeOperationsException, InstanceNotFoundException, InvalidTargetObjectTypeException
    {
-      if (resource == null) throw new RuntimeOperationsException(new IllegalArgumentException("Managed resource cannot be null"));
       if (!isResourceTypeSupported(resourceType)) throw new InvalidTargetObjectTypeException(resourceType);
 
       Logger logger = getLogger();
@@ -167,7 +169,7 @@ public class RequiredModelMBean implements ModelMBean, MBeanRegistration, Notifi
    private boolean isResourceTypeSupported(String resourceType)
    {
       // For now only object reference is supported
-      return OBJECT_RESOURCE_TYPE.equals(resourceType);
+      return OBJECT_RESOURCE_TYPE.equalsIgnoreCase(resourceType);
    }
 
    private Object getManagedResource()
@@ -182,10 +184,11 @@ public class RequiredModelMBean implements ModelMBean, MBeanRegistration, Notifi
 
    public void addAttributeChangeNotificationListener(NotificationListener listener, String attributeName, Object handback) throws MBeanException, RuntimeOperationsException, IllegalArgumentException
    {
-      if (listener == null) throw new RuntimeOperationsException(new IllegalArgumentException("Listener cannot be null"));
+      if (listener == null) throw new IllegalArgumentException("Listener cannot be null");
       AttributeChangeNotificationFilter filter = new AttributeChangeNotificationFilter();
       if (attributeName != null)
       {
+         if (m_modelMBeanInfo.getAttribute(attributeName) == null) throw new RuntimeOperationsException(new IllegalArgumentException("No attribute named " + attributeName));
          filter.enableAttribute(attributeName);
       }
       else
@@ -206,26 +209,54 @@ public class RequiredModelMBean implements ModelMBean, MBeanRegistration, Notifi
 
    public void addNotificationListener(NotificationListener listener, NotificationFilter filter, Object handback) throws IllegalArgumentException
    {
+      if (listener == null) throw new IllegalArgumentException("listener cannot be null");
       m_generalBroadcaster.addNotificationListener(listener, filter, handback);
    }
 
    public MBeanNotificationInfo[] getNotificationInfo()
    {
-      return m_modelMBeanInfo.getNotifications();
+      MBeanNotificationInfo[] notifications = m_modelMBeanInfo.getNotifications();
+      boolean hasGeneric = false;
+      boolean hasAttribute = false;
+      int toAdd = 2;
+      for (int i=0; i < notifications.length; i++) {
+         String name = notifications[i].getName();
+         if ("GENERIC".equals(name))
+         {
+            hasGeneric = true;
+            toAdd -= 1;
+            if (hasAttribute) break;
+         }
+         else if ("ATTRIBUTE_CHANGE".equals(name))
+         {
+            hasAttribute = true;
+            toAdd -= 1;
+            if (hasGeneric) break;
+         }
+      }
+
+      // clone the array, adding in missing notifications
+      MBeanNotificationInfo[] newNotifications = new MBeanNotificationInfo[notifications.length + toAdd];
+      System.arraycopy(notifications, 0, newNotifications, toAdd, notifications.length);
+      if (!hasAttribute) newNotifications[--toAdd] = ATTR_CHANGE_INFO;
+      if (!hasGeneric) newNotifications[--toAdd] = GENERIC_INFO;
+      return newNotifications;
    }
 
    public void removeAttributeChangeNotificationListener(NotificationListener listener, String attributeName) throws MBeanException, RuntimeOperationsException, ListenerNotFoundException
    {
+      //todo there may be a problem here if the original handback was not null - should we go directly to removeListener(NotificationListener) ?
       removeAttributeChangeNotificationListener(listener, attributeName, null);
    }
 
    // Not in the spec but needed
    private void removeAttributeChangeNotificationListener(NotificationListener listener, String attributeName, Object handback) throws MBeanException, RuntimeOperationsException, ListenerNotFoundException
    {
-      if (listener == null) throw new RuntimeOperationsException(new IllegalArgumentException("Listener cannot be null"));
+      if (listener == null) throw new ListenerNotFoundException("listener is null");
       AttributeChangeNotificationFilter filter = new AttributeChangeNotificationFilter();
       if (attributeName != null)
       {
+         if (m_modelMBeanInfo.getAttribute(attributeName) == null) throw new RuntimeOperationsException(new IllegalArgumentException("No attribute named " + attributeName));
          filter.enableAttribute(attributeName);
       }
       else
@@ -287,12 +318,14 @@ public class RequiredModelMBean implements ModelMBean, MBeanRegistration, Notifi
 
    public void sendNotification(String message) throws MBeanException, RuntimeOperationsException
    {
-      Notification notification = new Notification("jmx.modelmbean.general", this, 1, message);
+      if (message == null) throw new RuntimeOperationsException(new IllegalArgumentException("message is null"));
+      Notification notification = new Notification("jmx.modelmbean.generic", this, 1, message);
       sendNotification(notification);
    }
 
    public void sendNotification(Notification notification) throws MBeanException, RuntimeOperationsException
    {
+      if (notification == null) throw new RuntimeOperationsException(new IllegalArgumentException("notification is null"));
       m_generalBroadcaster.sendNotification(notification);
    }
 
@@ -1094,7 +1127,7 @@ public class RequiredModelMBean implements ModelMBean, MBeanRegistration, Notifi
 
    private NotificationBroadcasterSupport getAttributeChangeBroadcaster()
    {
-      return m_attributeChangeBroadcaster;
+      return m_generalBroadcaster;
    }
 
    private MBeanServer getMBeanServer()
